#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab

import logging
import os
import shutil
import subprocess
import sys
from socket import getfqdn
from poaupdater import uConfig, uLogging
from oam_common import endpoint_key, endpoint_cert

KEY_PATH = os.path.join('/etc/pki/tls/private', endpoint_key)
CERT_PATH = os.path.join('/etc/pki/tls/certs', endpoint_cert)


def generate_certificate():
    logging.info('Generating OAM certificate...')
    if os.path.isfile(CERT_PATH):
        # Ensure correct file permissions set.
        os.chmod(KEY_PATH, 0640)
        os.chmod(CERT_PATH, 0644)
    else:
        subprocess.call(['openssl', 'genrsa', '-out', endpoint_key], stderr=sys.stderr)
        if not os.path.isfile(endpoint_key):
            raise Exception("SSL private key generation failed.")

        hostname = getfqdn()
        subprocess.call(['openssl', 'req', '-x509', '-new', '-nodes', '-key', endpoint_key, '-out', endpoint_cert,
                         '-subj', '/DC=APS/DC=Application Endpoint/O=%s/OU=APS/CN=%s' % (hostname, hostname)],
                        stderr=sys.stderr)
        if not os.path.isfile(endpoint_cert):
            raise Exception("SSL certificate generation failed.")

        os.chmod(endpoint_key, 0640)
        os.chmod(endpoint_cert, 0644)

        shutil.move(endpoint_key, KEY_PATH)
        shutil.move(endpoint_cert, CERT_PATH)

    logging.info('The OAM certificate was successfully generated...')


def main():
    config = uConfig.Config()
    uLogging.init(config)
    logging.basicConfig(level=logging.INFO)
    logging.getLogger().addHandler(uLogging.Handler())

    generate_certificate()


if __name__ == "__main__":
    main()
